@@ -1,10 +1,54 @@
+//! Interact with the [`TestRegistry`]
+//!
+//! # Example
+//!
+//! ```no_run
+//! use cargo_test_support::registry::Package;
+//! use cargo_test_support::project;
+//! use cargo_test_support::str;
+//!
+//! // Publish package "a" depending on "b".
+//! Package::new("a", "1.0.0")
+//!     .dep("b", "1.0.0")
+//!     .file("src/lib.rs", r#"
+//!         extern crate b;
+//!         pub fn f() -> i32 { b::f() * 2 }
+//!     "#)
+//!     .publish();
+//!
+//! // Publish package "b".
+//! Package::new("b", "1.0.0")
+//!     .file("src/lib.rs", r#"
+//!         pub fn f() -> i32 { 12 }
+//!     "#)
+//!     .publish();
+//!
+//! // Create a project that uses package "a".
+//! let p = project()
+//!     .file("Cargo.toml", r#"
+//!         [package]
+//!         name = "foo"
+//!         version = "0.0.1"
+//!
+//!         [dependencies]
+//!         a = "1.0"
+//!     "#)
+//!     .file("src/main.rs", r#"
+//!         extern crate a;
+//!         fn main() { println!("{}", a::f()); }
+//!     "#)
+//!     .build();
+//!
+//! // p.cargo("run").with_stdout_data(str!["24"]).run();
+//! ```
+
 use crate::git::repo;
 use crate::paths;
 use crate::publish::{create_index_line, write_to_index};
-use cargo_util::paths::append;
 use cargo_util::Sha256;
-use flate2::write::GzEncoder;
+use cargo_util::paths::append;
 use flate2::Compression;
+use flate2::write::GzEncoder;
 use pasetors::keys::{AsymmetricPublicKey, AsymmetricSecretKey};
 use pasetors::paserk::FormatAsPaserk;
 use pasetors::token::UntrustedToken;
@@ -20,39 +64,64 @@ use time::format_description::well_known::Rfc3339;
 use time::{Duration, OffsetDateTime};
 use url::Url;
 
-/// Gets the path to the local index pretending to be crates.io. This is a Git repo
+/// Path to the local index for pseudo-crates.io.
+///
+/// This is a Git repo
 /// initialized with a `config.json` file pointing to `dl_path` for downloads
 /// and `api_path` for uploads.
+///
+/// ex: `$CARGO_TARGET_TMPDIR/cit/t0/registry`
 pub fn registry_path() -> PathBuf {
     generate_path("registry")
 }
-/// Gets the path for local web API uploads. Cargo will place the contents of a web API
+
+/// Path to the local web API uploads
+///
+/// Cargo will place the contents of a web API
 /// request here. For example, `api/v1/crates/new` is the result of publishing a crate.
+///
+/// ex: `$CARGO_TARGET_TMPDIR/cit/t0/api`
 pub fn api_path() -> PathBuf {
     generate_path("api")
 }
-/// Gets the path where crates can be downloaded using the web API endpoint. Crates
+
+/// Path to download `.crate` files using the web API endpoint.
+///
+/// Crates
 /// should be organized as `{name}/{version}/download` to match the web API
 /// endpoint. This is rarely used and must be manually set up.
-fn dl_path() -> PathBuf {
+///
+/// ex: `$CARGO_TARGET_TMPDIR/cit/t0/dl`
+pub fn dl_path() -> PathBuf {
     generate_path("dl")
 }
-/// Gets the alternative-registry version of `registry_path`.
-fn alt_registry_path() -> PathBuf {
+
+/// Path to the alternative-registry version of [`registry_path`]
+///
+/// ex: `$CARGO_TARGET_TMPDIR/cit/t0/alternative-registry`
+pub fn alt_registry_path() -> PathBuf {
     generate_path("alternative-registry")
 }
-/// Gets the alternative-registry version of `registry_url`.
+
+/// URL to the alternative-registry version of `registry_url`
 fn alt_registry_url() -> Url {
     generate_url("alternative-registry")
 }
-/// Gets the alternative-registry version of `dl_path`.
+
+/// Path to the alternative-registry version of [`dl_path`]
+///
+/// ex: `$CARGO_TARGET_TMPDIR/cit/t0/alternative-dl`
 pub fn alt_dl_path() -> PathBuf {
     generate_path("alternative-dl")
 }
-/// Gets the alternative-registry version of `api_path`.
+
+/// Path to the alternative-registry version of [`api_path`]
+///
+/// ex: `$CARGO_TARGET_TMPDIR/cit/t0/alternative-api`
 pub fn alt_api_path() -> PathBuf {
     generate_path("alternative-api")
 }
+
 fn generate_path(name: &str) -> PathBuf {
     paths::root().join(name)
 }
@@ -60,6 +129,7 @@ fn generate_url(name: &str) -> Url {
     Url::from_file_path(generate_path(name)).ok().unwrap()
 }
 
+/// Auth-token for publishing, see [`RegistryBuilder::token`]
 #[derive(Clone)]
 pub enum Token {
     Plaintext(String),
@@ -68,6 +138,7 @@ pub enum Token {
 
 impl Token {
     /// This is a valid PASETO secret key.
+    ///
     /// This one is already publicly available as part of the text of the RFC so is safe to use for tests.
     pub fn rfc_key() -> Token {
         Token::Keys(
@@ -80,7 +151,9 @@ impl Token {
 
 type RequestCallback = Box<dyn Send + Fn(&Request, &HttpServer) -> Response>;
 
-/// A builder for initializing registries.
+/// Prepare a local [`TestRegistry`] fixture
+///
+/// See also [`init`] and [`alt_init`]
 pub struct RegistryBuilder {
     /// If set, configures an alternate registry with the given name.
     alternative: Option<String>,
@@ -104,8 +177,13 @@ pub struct RegistryBuilder {
     not_found_handler: RequestCallback,
     /// If nonzero, the git index update to be delayed by the given number of seconds.
     delayed_index_update: usize,
+    /// Credential provider in configuration
+    credential_provider: Option<String>,
 }
 
+/// A local registry fixture
+///
+/// Most tests won't need to call this directly but instead interact with [`Package`]
 pub struct TestRegistry {
     server: Option<HttpServerHandle>,
     index_url: Url,
@@ -172,6 +250,7 @@ impl RegistryBuilder {
             custom_responders: HashMap::new(),
             not_found_handler: Box::new(not_found),
             delayed_index_update: 0,
+            credential_provider: None,
         }
     }
 
@@ -203,14 +282,14 @@ impl RegistryBuilder {
         self
     }
 
-    /// Sets whether or not to initialize as an alternative registry.
+    /// Initializes as an alternative registry with the given name.
     #[must_use]
     pub fn alternative_named(mut self, alt: &str) -> Self {
         self.alternative = Some(alt.to_string());
         self
     }
 
-    /// Sets whether or not to initialize as an alternative registry.
+    /// Initializes as an alternative registry named "alternative".
     #[must_use]
     pub fn alternative(self) -> Self {
         self.alternative_named("alternative")
@@ -266,10 +345,17 @@ impl RegistryBuilder {
         self
     }
 
+    /// The credential provider to configure for this registry.
+    #[must_use]
+    pub fn credential_provider(mut self, provider: &[&str]) -> Self {
+        self.credential_provider = Some(format!("['{}']", provider.join("','")));
+        self
+    }
+
     /// Initializes the registry.
     #[must_use]
     pub fn build(self) -> TestRegistry {
-        let config_path = paths::home().join(".cargo/config");
+        let config_path = paths::cargo_home().join("config.toml");
         t!(fs::create_dir_all(config_path.parent().unwrap()));
         let prefix = if let Some(alternative) = &self.alternative {
             format!("{alternative}-")
@@ -336,6 +422,18 @@ impl RegistryBuilder {
                     .as_bytes(),
                 )
                 .unwrap();
+                if let Some(p) = &self.credential_provider {
+                    append(
+                        &config_path,
+                        &format!(
+                            "
+                        credential-provider = {p}
+                        "
+                        )
+                        .as_bytes(),
+                    )
+                    .unwrap()
+                }
             } else {
                 append(
                     &config_path,
@@ -351,11 +449,25 @@ impl RegistryBuilder {
                     .as_bytes(),
                 )
                 .unwrap();
+
+                if let Some(p) = &self.credential_provider {
+                    append(
+                        &config_path,
+                        &format!(
+                            "
+                        [registry]
+                        credential-provider = {p}
+                        "
+                        )
+                        .as_bytes(),
+                    )
+                    .unwrap()
+                }
             }
         }
 
         if self.configure_token {
-            let credentials = paths::home().join(".cargo/credentials.toml");
+            let credentials = paths::cargo_home().join("credentials.toml");
             match &registry.token {
                 Token::Plaintext(token) => {
                     if let Some(alternative) = &self.alternative {
@@ -423,68 +535,31 @@ impl RegistryBuilder {
     }
 }
 
-/// A builder for creating a new package in a registry.
+/// Published package builder for [`TestRegistry`]
 ///
 /// This uses "source replacement" using an automatically generated
 /// `.cargo/config` file to ensure that dependencies will use these packages
 /// instead of contacting crates.io. See `source-replacement.md` for more
 /// details on how source replacement works.
 ///
-/// Call `publish` to finalize and create the package.
+/// Call [`Package::publish`] to finalize and create the package.
 ///
 /// If no files are specified, an empty `lib.rs` file is automatically created.
 ///
 /// The `Cargo.toml` file is automatically generated based on the methods
-/// called on `Package` (for example, calling `dep()` will add to the
+/// called on `Package` (for example, calling [`Package::dep()`] will add to the
 /// `[dependencies]` automatically). You may also specify a `Cargo.toml` file
 /// to override the generated one.
 ///
 /// This supports different registry types:
 /// - Regular source replacement that replaces `crates.io` (the default).
 /// - A "local registry" which is a subset for vendoring (see
-///   `Package::local`).
+///   [`Package::local`]).
 /// - An "alternative registry" which requires specifying the registry name
-///   (see `Package::alternative`).
+///   (see [`Package::alternative`]).
 ///
 /// This does not support "directory sources". See `directory.rs` for
 /// `VendorPackage` which implements directory sources.
-///
-/// # Example
-/// ```
-/// // Publish package "a" depending on "b".
-/// Package::new("a", "1.0.0")
-///     .dep("b", "1.0.0")
-///     .file("src/lib.rs", r#"
-///         extern crate b;
-///         pub fn f() -> i32 { b::f() * 2 }
-///     "#)
-///     .publish();
-///
-/// // Publish package "b".
-/// Package::new("b", "1.0.0")
-///     .file("src/lib.rs", r#"
-///         pub fn f() -> i32 { 12 }
-///     "#)
-///     .publish();
-///
-/// // Create a project that uses package "a".
-/// let p = project()
-///     .file("Cargo.toml", r#"
-///         [package]
-///         name = "foo"
-///         version = "0.0.1"
-///
-///         [dependencies]
-///         a = "1.0"
-///     "#)
-///     .file("src/main.rs", r#"
-///         extern crate a;
-///         fn main() { println!("{}", a::f()); }
-///     "#)
-///     .build();
-///
-/// p.cargo("run").with_stdout("24").run();
-/// ```
 #[must_use]
 pub struct Package {
     name: String,
@@ -495,27 +570,36 @@ pub struct Package {
     features: FeatureMap,
     local: bool,
     alternative: bool,
-    invalid_json: bool,
+    invalid_index_line: bool,
+    index_line: Option<String>,
+    edition: Option<String>,
+    resolver: Option<String>,
     proc_macro: bool,
     links: Option<String>,
     rust_version: Option<String>,
     cargo_features: Vec<String>,
+    pubtime: Option<String>,
     v: Option<u32>,
 }
 
 pub(crate) type FeatureMap = BTreeMap<String, Vec<String>>;
 
+/// Published package dependency builder, see [`Package::add_dep`]
 #[derive(Clone)]
 pub struct Dependency {
     name: String,
     vers: String,
     kind: String,
-    artifact: Option<(String, Option<String>)>,
+    artifact: Option<String>,
+    bindep_target: Option<String>,
+    lib: bool,
     target: Option<String>,
     features: Vec<String>,
     registry: Option<String>,
     package: Option<String>,
     optional: bool,
+    default_features: bool,
+    public: bool,
 }
 
 /// Entry with data that corresponds to [`tar::EntryType`].
@@ -539,14 +623,18 @@ struct PackageFile {
 
 const DEFAULT_MODE: u32 = 0o644;
 
-/// Initializes the on-disk registry and sets up the config so that crates.io
-/// is replaced with the one on disk.
+/// Setup a local pseudo-crates.io [`TestRegistry`]
+///
+/// This is implicitly called by [`Package::new`].
+///
+/// When calling `cargo publish`, see instead [`crate::publish`].
 pub fn init() -> TestRegistry {
     RegistryBuilder::new().build()
 }
 
-/// Variant of `init` that initializes the "alternative" registry and crates.io
-/// replacement.
+/// Setup a local "alternative" [`TestRegistry`]
+///
+/// When calling `cargo publish`, see instead [`crate::publish`].
 pub fn alt_init() -> TestRegistry {
     init();
     RegistryBuilder::new().alternative().build()
@@ -559,15 +647,15 @@ pub struct HttpServerHandle {
 
 impl HttpServerHandle {
     pub fn index_url(&self) -> Url {
-        Url::parse(&format!("sparse+http://{}/index/", self.addr.to_string())).unwrap()
+        Url::parse(&format!("sparse+http://{}/index/", self.addr)).unwrap()
     }
 
     pub fn api_url(&self) -> Url {
-        Url::parse(&format!("http://{}/", self.addr.to_string())).unwrap()
+        Url::parse(&format!("http://{}/", self.addr)).unwrap()
     }
 
     pub fn dl_url(&self) -> Url {
-        Url::parse(&format!("http://{}/dl", self.addr.to_string())).unwrap()
+        Url::parse(&format!("http://{}/dl", self.addr)).unwrap()
     }
 
     fn stop(&self) {
@@ -740,6 +828,7 @@ impl HttpServer {
             let buf = buf.get_mut();
             write!(buf, "HTTP/1.1 {}\r\n", response.code).unwrap();
             write!(buf, "Content-Length: {}\r\n", response.body.len()).unwrap();
+            write!(buf, "Connection: close\r\n").unwrap();
             for header in response.headers {
                 write!(buf, "{}\r\n", header).unwrap();
             }
@@ -749,7 +838,7 @@ impl HttpServer {
         }
     }
 
-    fn check_authorized(&self, req: &Request, mutation: Option<Mutation>) -> bool {
+    fn check_authorized(&self, req: &Request, mutation: Option<Mutation<'_>>) -> bool {
         let (private_key, private_key_subject) = if mutation.is_some() || self.auth_required {
             match &self.token {
                 Token::Plaintext(token) => return Some(token) == req.authorization.as_ref(),
@@ -791,7 +880,8 @@ impl HttpServer {
             url: &'a str,
             kip: &'a str,
         }
-        let footer: Footer = t!(serde_json::from_slice(untrusted_token.untrusted_footer()).ok());
+        let footer: Footer<'_> =
+            t!(serde_json::from_slice(untrusted_token.untrusted_footer()).ok());
         if footer.kip != paserk_pub_key_id {
             return false;
         }
@@ -803,9 +893,8 @@ impl HttpServer {
 
         // - The URL matches the registry base URL
         if footer.url != "https://github.com/rust-lang/crates.io-index"
-            && footer.url != &format!("sparse+http://{}/index/", self.addr.to_string())
+            && footer.url != &format!("sparse+http://{}/index/", self.addr)
         {
-            dbg!(footer.url);
             return false;
         }
 
@@ -821,20 +910,18 @@ impl HttpServer {
             _challenge: Option<&'a str>, // todo: PASETO with challenges
             v: Option<u8>,
         }
-        let message: Message = t!(serde_json::from_str(trusted_token.payload()).ok());
+        let message: Message<'_> = t!(serde_json::from_str(trusted_token.payload()).ok());
         let token_time = t!(OffsetDateTime::parse(message.iat, &Rfc3339).ok());
         let now = OffsetDateTime::now_utc();
         if (now - token_time) > Duration::MINUTE {
             return false;
         }
         if private_key_subject.as_deref() != message.sub {
-            dbg!(message.sub);
             return false;
         }
         // - If the claim v is set, that it has the value of 1.
         if let Some(v) = message.v {
             if v != 1 {
-                dbg!(message.v);
                 return false;
             }
         }
@@ -844,22 +931,18 @@ impl HttpServer {
         if let Some(mutation) = mutation {
             //  - That the operation matches the mutation field and is one of publish, yank, or unyank.
             if message.mutation != Some(mutation.mutation) {
-                dbg!(message.mutation);
                 return false;
             }
             //  - That the package, and version match the request.
             if message.name != mutation.name {
-                dbg!(message.name);
                 return false;
             }
             if message.vers != mutation.vers {
-                dbg!(message.vers);
                 return false;
             }
             //  - If the mutation is publish, that the version has not already been published, and that the hash matches the request.
             if mutation.mutation == "publish" {
                 if message.cksum != mutation.cksum {
-                    dbg!(message.cksum);
                     return false;
                 }
             }
@@ -975,6 +1058,19 @@ impl HttpServer {
         }
     }
 
+    /// Return too many requests (HTTP 429)
+    pub fn too_many_requests(&self, _req: &Request, delay: std::time::Duration) -> Response {
+        Response {
+            code: 429,
+            headers: vec![format!("Retry-After: {}", delay.as_secs())],
+            body: format!(
+                "too many requests, try again in {} seconds",
+                delay.as_secs()
+            )
+            .into_bytes(),
+        }
+    }
+
     /// Serve the download endpoint
     pub fn dl(&self, req: &Request) -> Response {
         let file = self
@@ -1125,12 +1221,15 @@ fn save_new_crate(
                 "name": name,
                 "req": dep.version_req,
                 "features": dep.features,
-                "default_features": true,
+                "default_features": dep.default_features,
                 "target": dep.target,
                 "optional": dep.optional,
                 "kind": dep.kind,
                 "registry": dep.registry,
                 "package": package,
+                "artifact": dep.artifact,
+                "bindep_target": dep.bindep_target,
+                "lib": dep.lib,
             })
         })
         .collect::<Vec<_>>();
@@ -1143,6 +1242,8 @@ fn save_new_crate(
         new_crate.features,
         false,
         new_crate.links,
+        new_crate.rust_version.as_deref(),
+        None,
         None,
     );
 
@@ -1153,7 +1254,7 @@ impl Package {
     /// Creates a new package builder.
     /// Call `publish()` to finalize and build the package.
     pub fn new(name: &str, vers: &str) -> Package {
-        let config = paths::home().join(".cargo/config");
+        let config = paths::cargo_home().join("config.toml");
         if !config.exists() {
             init();
         }
@@ -1166,11 +1267,15 @@ impl Package {
             features: BTreeMap::new(),
             local: false,
             alternative: false,
-            invalid_json: false,
+            invalid_index_line: false,
+            index_line: None,
+            edition: None,
+            resolver: None,
             proc_macro: false,
             links: None,
             rust_version: None,
             cargo_features: Vec::new(),
+            pubtime: None,
             v: None,
         }
     }
@@ -1192,6 +1297,8 @@ impl Package {
     /// See `src/doc/src/reference/registries.md` for more details on
     /// alternative registries. See `alt_registry.rs` for the tests that use
     /// this.
+    ///
+    /// **Requires:** [`alt_init`]
     pub fn alternative(&mut self, alternative: bool) -> &mut Package {
         self.alternative = alternative;
         self
@@ -1240,7 +1347,7 @@ impl Package {
     }
 
     /// Adds a normal dependency. Example:
-    /// ```
+    /// ```toml
     /// [dependencies]
     /// foo = {version = "1.0"}
     /// ```
@@ -1249,7 +1356,7 @@ impl Package {
     }
 
     /// Adds a dependency with the given feature. Example:
-    /// ```
+    /// ```toml
     /// [dependencies]
     /// foo = {version = "1.0", "features": ["feat1", "feat2"]}
     /// ```
@@ -1272,7 +1379,7 @@ impl Package {
     }
 
     /// Adds a dev-dependency. Example:
-    /// ```
+    /// ```toml
     /// [dev-dependencies]
     /// foo = {version = "1.0"}
     /// ```
@@ -1281,7 +1388,7 @@ impl Package {
     }
 
     /// Adds a build-dependency. Example:
-    /// ```
+    /// ```toml
     /// [build-dependencies]
     /// foo = {version = "1.0"}
     /// ```
@@ -1300,6 +1407,18 @@ impl Package {
         self
     }
 
+    /// Specifies `package.edition`
+    pub fn edition(&mut self, edition: &str) -> &mut Package {
+        self.edition = Some(edition.to_owned());
+        self
+    }
+
+    /// Specifies `package.resolver`
+    pub fn resolver(&mut self, resolver: &str) -> &mut Package {
+        self.resolver = Some(resolver.to_owned());
+        self
+    }
+
     /// Specifies whether or not this is a proc macro.
     pub fn proc_macro(&mut self, proc_macro: bool) -> &mut Package {
         self.proc_macro = proc_macro;
@@ -1321,8 +1440,16 @@ impl Package {
 
     /// Causes the JSON line emitted in the index to be invalid, presumably
     /// causing Cargo to skip over this version.
-    pub fn invalid_json(&mut self, invalid: bool) -> &mut Package {
-        self.invalid_json = invalid;
+    pub fn invalid_index_line(&mut self, invalid: bool) -> &mut Package {
+        self.invalid_index_line = invalid;
+        self
+    }
+
+    /// Override the auto-generated index line
+    ///
+    /// This can give more control over error cases than [`Package::invalid_index_line`]
+    pub fn index_line(&mut self, line: &str) -> &mut Package {
+        self.index_line = Some(line.to_owned());
         self
     }
 
@@ -1336,9 +1463,15 @@ impl Package {
         self
     }
 
+    /// The publish time for the package in ISO8601 with UTC timezone (e.g. 2025-11-12T19:30:12Z)
+    pub fn pubtime(&mut self, time: &str) -> &mut Package {
+        self.pubtime = Some(time.to_owned());
+        self
+    }
+
     /// Sets the index schema version for this package.
     ///
-    /// See `cargo::sources::registry::RegistryPackage` for more information.
+    /// See `cargo::sources::registry::IndexPackage` for more information.
     pub fn schema_version(&mut self, version: u32) -> &mut Package {
         self.v = Some(version);
         self
@@ -1369,17 +1502,25 @@ impl Package {
                     (true, Some("alternative")) => None,
                     _ => panic!("registry_dep currently only supports `alternative`"),
                 };
+                let artifact = if let Some(artifact) = &dep.artifact {
+                    serde_json::json!([artifact])
+                } else {
+                    serde_json::json!(null)
+                };
                 serde_json::json!({
                     "name": dep.name,
                     "req": dep.vers,
                     "features": dep.features,
-                    "default_features": true,
+                    "default_features": dep.default_features,
                     "target": dep.target,
-                    "artifact": dep.artifact,
+                    "artifact": artifact,
+                    "bindep_target": dep.bindep_target,
+                    "lib": dep.lib,
                     "optional": dep.optional,
                     "kind": dep.kind,
                     "registry": registry_url,
                     "package": dep.package,
+                    "public": dep.public,
                 })
             })
             .collect::<Vec<_>>();
@@ -1387,21 +1528,27 @@ impl Package {
             let c = t!(fs::read(&self.archive_dst()));
             cksum(&c)
         };
-        let name = if self.invalid_json {
-            serde_json::json!(1)
+        let line = if let Some(line) = self.index_line.clone() {
+            line
         } else {
-            serde_json::json!(self.name)
+            let name = if self.invalid_index_line {
+                serde_json::json!(1)
+            } else {
+                serde_json::json!(self.name)
+            };
+            create_index_line(
+                name,
+                &self.vers,
+                deps,
+                &cksum,
+                self.features.clone(),
+                self.yanked,
+                self.links.clone(),
+                self.rust_version.as_deref(),
+                self.pubtime.as_deref(),
+                self.v,
+            )
         };
-        let line = create_index_line(
-            name,
-            &self.vers,
-            deps,
-            &cksum,
-            self.features.clone(),
-            self.yanked,
-            self.links.clone(),
-            self.v,
-        );
 
         let registry_path = if self.alternative {
             alt_registry_path()
@@ -1418,7 +1565,8 @@ impl Package {
         let dst = self.archive_dst();
         t!(fs::create_dir_all(dst.parent().unwrap()));
         let f = t!(File::create(&dst));
-        let mut a = Builder::new(GzEncoder::new(f, Compression::default()));
+        let mut a = Builder::new(GzEncoder::new(f, Compression::none()));
+        a.sparse(false);
 
         if !self
             .files
@@ -1475,7 +1623,39 @@ impl Package {
         ));
 
         if let Some(version) = &self.rust_version {
-            manifest.push_str(&format!("rust-version = \"{}\"", version));
+            manifest.push_str(&format!("rust-version = \"{}\"\n", version));
+        }
+
+        if let Some(edition) = &self.edition {
+            manifest.push_str(&format!("edition = \"{}\"\n", edition));
+        }
+
+        if let Some(resolver) = &self.resolver {
+            manifest.push_str(&format!("resolver = \"{}\"\n", resolver));
+        }
+
+        if !self.features.is_empty() {
+            let features: Vec<String> = self
+                .features
+                .iter()
+                .map(|(feature, features)| {
+                    if features.is_empty() {
+                        format!("{} = []", feature)
+                    } else {
+                        format!(
+                            "{} = [{}]",
+                            feature,
+                            features
+                                .iter()
+                                .map(|s| format!("\"{}\"", s))
+                                .collect::<Vec<_>>()
+                                .join(", ")
+                        )
+                    }
+                })
+                .collect();
+
+            manifest.push_str(&format!("\n[features]\n{}", features.join("\n")));
         }
 
         for dep in self.deps.iter() {
@@ -1495,16 +1675,37 @@ impl Package {
             "#,
                 target, kind, dep.name, dep.vers
             ));
-            if let Some((artifact, target)) = &dep.artifact {
+            if dep.optional {
+                manifest.push_str("optional = true\n");
+            }
+            if let Some(artifact) = &dep.artifact {
                 manifest.push_str(&format!("artifact = \"{}\"\n", artifact));
-                if let Some(target) = &target {
-                    manifest.push_str(&format!("target = \"{}\"\n", target))
-                }
+            }
+            if let Some(target) = &dep.bindep_target {
+                manifest.push_str(&format!("target = \"{}\"\n", target));
+            }
+            if dep.lib {
+                manifest.push_str("lib = true\n");
             }
             if let Some(registry) = &dep.registry {
                 assert_eq!(registry, "alternative");
                 manifest.push_str(&format!("registry-index = \"{}\"", alt_registry_url()));
             }
+            if !dep.default_features {
+                manifest.push_str("default-features = false\n");
+            }
+            if !dep.features.is_empty() {
+                let mut features = String::new();
+                serde::Serialize::serialize(
+                    &dep.features,
+                    toml::ser::ValueSerializer::new(&mut features),
+                )
+                .unwrap();
+                manifest.push_str(&format!("features = {}\n", features));
+            }
+            if let Some(package) = &dep.package {
+                manifest.push_str(&format!("package = \"{}\"\n", package));
+            }
         }
         if self.proc_macro {
             manifest.push_str("[lib]\nproc-macro = true\n");
@@ -1553,7 +1754,12 @@ impl Package {
     /// Returns the path to the compressed package file.
     pub fn archive_dst(&self) -> PathBuf {
         if self.local {
-            registry_path().join(format!("{}-{}.crate", self.name, self.vers))
+            let path = if self.alternative {
+                alt_registry_path()
+            } else {
+                registry_path()
+            };
+            path.join(format!("{}-{}.crate", self.name, self.vers))
         } else if self.alternative {
             alt_dl_path()
                 .join(&self.name)
@@ -1565,6 +1771,7 @@ impl Package {
     }
 }
 
+/// Generate a checksum
 pub fn cksum(s: &[u8]) -> String {
     Sha256::new().update(s).finish_hex()
 }
@@ -1576,11 +1783,15 @@ impl Dependency {
             vers: vers.to_string(),
             kind: "normal".to_string(),
             artifact: None,
+            bindep_target: None,
+            lib: false,
             target: None,
             features: Vec::new(),
             package: None,
             optional: false,
             registry: None,
+            default_features: true,
+            public: false,
         }
     }
 
@@ -1605,7 +1816,8 @@ impl Dependency {
     /// Change the artifact to be of the given kind, like "bin", or "staticlib",
     /// along with a specific target triple if provided.
     pub fn artifact(&mut self, kind: &str, target: Option<String>) -> &mut Self {
-        self.artifact = Some((kind.to_string(), target));
+        self.artifact = Some(kind.to_string());
+        self.bindep_target = target;
         self
     }
 
@@ -1632,4 +1844,16 @@ impl Dependency {
         self.optional = optional;
         self
     }
+
+    /// Changes this to an public dependency.
+    pub fn public(&mut self, public: bool) -> &mut Self {
+        self.public = public;
+        self
+    }
+
+    /// Adds `default-features = false` if the argument is `false`.
+    pub fn default_features(&mut self, default_features: bool) -> &mut Self {
+        self.default_features = default_features;
+        self
+    }
 }
