@@ -1,15 +1,17 @@
 //! Tests for local-registry sources.
 
-use cargo_test_support::paths::{self, CargoPathExt};
-use cargo_test_support::registry::{registry_path, Package};
-use cargo_test_support::{basic_manifest, project, t};
 use std::fs;
 
+use crate::prelude::*;
+use cargo_test_support::paths;
+use cargo_test_support::registry::{Package, registry_path};
+use cargo_test_support::{basic_manifest, project, str, t};
+
 fn setup() {
     let root = paths::root();
     t!(fs::create_dir(&root.join(".cargo")));
     t!(fs::write(
-        root.join(".cargo/config"),
+        root.join(".cargo/config.toml"),
         r#"
             [source.crates-io]
             registry = 'https://wut'
@@ -33,9 +35,10 @@ fn simple() {
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [dependencies]
@@ -49,19 +52,62 @@ fn simple() {
         .build();
 
     p.cargo("build")
-        .with_stderr(
-            "\
-[UNPACKING] bar v0.0.1 ([..])
+        .with_stderr_data(str![[r#"
+[LOCKING] 1 package to latest compatible version
+[UNPACKING] bar v0.0.1 (registry `[ROOT]/registry`)
 [COMPILING] bar v0.0.1
-[COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] [..]
-",
-        )
+[COMPILING] foo v0.0.1 ([ROOT]/foo)
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+
+"#]])
+        .run();
+    p.cargo("build")
+        .with_stderr_data(str![[r#"
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+
+"#]])
         .run();
-    p.cargo("build").with_stderr("[FINISHED] [..]").run();
     p.cargo("test").run();
 }
 
+#[cargo_test]
+fn not_found() {
+    setup();
+    // Publish a package so that the directory hierarchy is created.
+    // Note, however, that we declare a dependency on baz.
+    Package::new("bar", "0.0.1").local(true).publish();
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                edition = "2015"
+                authors = []
+
+                [dependencies]
+                baz = "0.0.1"
+            "#,
+        )
+        .file(
+            "src/lib.rs",
+            "extern crate baz; pub fn foo() { baz::bar(); }",
+        )
+        .build();
+
+    p.cargo("check")
+        .with_status(101)
+        .with_stderr_data(str![[r#"
+[ERROR] no matching package named `baz` found
+location searched: `[ROOT]/registry` index (which is replacing registry `crates-io`)
+required by package `foo v0.0.1 ([ROOT]/foo)`
+
+"#]])
+        .run();
+}
+
 #[cargo_test]
 fn depend_on_yanked() {
     setup();
@@ -71,9 +117,10 @@ fn depend_on_yanked() {
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [dependencies]
@@ -93,11 +140,10 @@ fn depend_on_yanked() {
         .publish();
 
     p.cargo("check")
-        .with_stderr(
-            "\
-[FINISHED] [..]
-",
-        )
+        .with_stderr_data(str![[r#"
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+
+"#]])
         .run();
 }
 
@@ -114,9 +160,10 @@ fn multiple_versions() {
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [dependencies]
@@ -129,15 +176,15 @@ fn multiple_versions() {
         )
         .build();
 
-    p.cargo("build")
-        .with_stderr(
-            "\
-[UNPACKING] bar v0.1.0 ([..])
-[COMPILING] bar v0.1.0
-[COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] [..]
-",
-        )
+    p.cargo("check")
+        .with_stderr_data(str![[r#"
+[LOCKING] 1 package to latest compatible version
+[UNPACKING] bar v0.1.0 (registry `[ROOT]/registry`)
+[CHECKING] bar v0.1.0
+[CHECKING] foo v0.0.1 ([ROOT]/foo)
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+
+"#]])
         .run();
 
     Package::new("bar", "0.2.0")
@@ -145,8 +192,12 @@ fn multiple_versions() {
         .file("src/lib.rs", "pub fn bar() {}")
         .publish();
 
-    p.cargo("update -v")
-        .with_stderr("[UPDATING] bar v0.1.0 -> v0.2.0")
+    p.cargo("update")
+        .with_stderr_data(str![[r#"
+[LOCKING] 1 package to latest compatible version
+[UPDATING] bar v0.1.0 -> v0.2.0
+
+"#]])
         .run();
 }
 
@@ -166,9 +217,10 @@ fn multiple_names() {
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [dependencies]
@@ -189,16 +241,19 @@ fn multiple_names() {
         )
         .build();
 
-    p.cargo("build")
-        .with_stderr(
-            "\
-[UNPACKING] [..]
-[UNPACKING] [..]
-[COMPILING] [..]
-[COMPILING] [..]
-[COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] [..]
-",
+    p.cargo("check")
+        .with_stderr_data(
+            str![[r#"
+[LOCKING] 2 packages to latest compatible versions
+[UNPACKING] bar v0.0.1 (registry `[ROOT]/registry`)
+[UNPACKING] baz v0.1.0 (registry `[ROOT]/registry`)
+[CHECKING] bar v0.0.1
+[CHECKING] baz v0.1.0
+[CHECKING] foo v0.0.1 ([ROOT]/foo)
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+
+"#]]
+            .unordered(),
         )
         .run();
 }
@@ -220,9 +275,10 @@ fn interdependent() {
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [dependencies]
@@ -243,17 +299,17 @@ fn interdependent() {
         )
         .build();
 
-    p.cargo("build")
-        .with_stderr(
-            "\
-[UNPACKING] [..]
-[UNPACKING] [..]
-[COMPILING] bar v0.0.1
-[COMPILING] baz v0.1.0
-[COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] [..]
-",
-        )
+    p.cargo("check")
+        .with_stderr_data(str![[r#"
+[LOCKING] 2 packages to latest compatible versions
+[UNPACKING] bar v0.0.1 (registry `[ROOT]/registry`)
+[UNPACKING] baz v0.1.0 (registry `[ROOT]/registry`)
+[CHECKING] bar v0.0.1
+[CHECKING] baz v0.1.0
+[CHECKING] foo v0.0.1 ([ROOT]/foo)
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+
+"#]])
         .run();
 }
 
@@ -270,9 +326,10 @@ fn path_dep_rewritten() {
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "baz"
                 version = "0.1.0"
+                edition = "2015"
                 authors = []
 
                 [dependencies]
@@ -288,9 +345,10 @@ fn path_dep_rewritten() {
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [dependencies]
@@ -311,17 +369,17 @@ fn path_dep_rewritten() {
         )
         .build();
 
-    p.cargo("build")
-        .with_stderr(
-            "\
-[UNPACKING] [..]
-[UNPACKING] [..]
-[COMPILING] bar v0.0.1
-[COMPILING] baz v0.1.0
-[COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] [..]
-",
-        )
+    p.cargo("check")
+        .with_stderr_data(str![[r#"
+[LOCKING] 2 packages to latest compatible versions
+[UNPACKING] bar v0.0.1 (registry `[ROOT]/registry`)
+[UNPACKING] baz v0.1.0 (registry `[ROOT]/registry`)
+[CHECKING] bar v0.0.1
+[CHECKING] baz v0.1.0
+[CHECKING] foo v0.0.1 ([ROOT]/foo)
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+
+"#]])
         .run();
 }
 
@@ -332,9 +390,10 @@ fn invalid_dir_bad() {
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [dependencies]
@@ -343,7 +402,7 @@ fn invalid_dir_bad() {
         )
         .file("src/lib.rs", "")
         .file(
-            ".cargo/config",
+            ".cargo/config.toml",
             r#"
                 [source.crates-io]
                 registry = 'https://wut'
@@ -355,25 +414,24 @@ fn invalid_dir_bad() {
         )
         .build();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
-        .with_stderr(
-            "\
-[ERROR] failed to get `bar` as a dependency of package `foo v0.0.1 [..]`
+        .with_stderr_data(str![[r#"
+[ERROR] failed to get `bar` as a dependency of package `foo v0.0.1 ([ROOT]/foo)`
 
 Caused by:
   failed to load source for dependency `bar`
 
 Caused by:
-  Unable to update registry `https://[..]`
+  Unable to update registry `crates-io`
 
 Caused by:
-  failed to update replaced source registry `https://[..]`
+  failed to update replaced source registry `crates-io`
 
 Caused by:
   local registry path is not a directory: [..]path[..]to[..]nowhere
-",
-        )
+
+"#]])
         .run();
 }
 
@@ -391,9 +449,10 @@ fn different_directory_replacing_the_registry_is_bad() {
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [dependencies]
@@ -405,7 +464,7 @@ fn different_directory_replacing_the_registry_is_bad() {
 
     // Generate a lock file against the crates.io registry
     Package::new("bar", "0.0.1").publish();
-    p.cargo("build").run();
+    p.cargo("check").run();
 
     // Switch back to our directory source, and now that we're replacing
     // crates.io make sure that this fails because we're replacing with a
@@ -417,10 +476,9 @@ fn different_directory_replacing_the_registry_is_bad() {
         .local(true)
         .publish();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
-        .with_stderr(
-            "\
+        .with_stderr_data(str![[r#"
 [ERROR] checksum for `bar v0.0.1` changed between lock files
 
 this could be indicative of a few possible errors:
@@ -431,8 +489,8 @@ this could be indicative of a few possible errors:
 
 unable to verify that `bar v0.0.1` is the same as when the lockfile was generated
 
-",
-        )
+
+"#]])
         .run();
 }
 
@@ -441,7 +499,7 @@ fn crates_io_registry_url_is_optional() {
     let root = paths::root();
     t!(fs::create_dir(&root.join(".cargo")));
     t!(fs::write(
-        root.join(".cargo/config"),
+        root.join(".cargo/config.toml"),
         r#"
             [source.crates-io]
             replace-with = 'my-awesome-local-registry'
@@ -460,9 +518,10 @@ fn crates_io_registry_url_is_optional() {
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [dependencies]
@@ -476,15 +535,20 @@ fn crates_io_registry_url_is_optional() {
         .build();
 
     p.cargo("build")
-        .with_stderr(
-            "\
-[UNPACKING] bar v0.0.1 ([..])
+        .with_stderr_data(str![[r#"
+[LOCKING] 1 package to latest compatible version
+[UNPACKING] bar v0.0.1 (registry `[ROOT]/registry`)
 [COMPILING] bar v0.0.1
-[COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] [..]
-",
-        )
+[COMPILING] foo v0.0.1 ([ROOT]/foo)
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+
+"#]])
+        .run();
+    p.cargo("build")
+        .with_stderr_data(str![[r#"
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+
+"#]])
         .run();
-    p.cargo("build").with_stderr("[FINISHED] [..]").run();
     p.cargo("test").run();
 }
